/*
** ----------------------------------------------------------------------
** SEGYLIB.C: LIBRARY OF ROUTINES USED TO ACCESS SEG-Y FORMAT DISK IMAGE
**            SEISMIC DATA
**
**              VERSION: V1.0
**          ORIGIN DATE: 5/14/92
**             COMPILER: BORLAND C/C++ V3.0
**
** MODIFICATION HISTORY: 10/30/92 - added Get4Bytes function
**                       10/30/92 - added Get2Bytes function
**                       10/30/92 - GetCDP argument X changed from int
**                                  to long
**                       11/4/92  - GoToByte argument byteNumber changed
**                                  from unsigned long to long
**
**               AUTHOR: F.N. ZIHLMAN, USGS
**                       BRANCH OF PETROLEUM GEOLOGY, GEOPHYSICS GROUP
**                       P.O. BOX 25046, MS 960
**                       DENVER, CO 80225
**                       303-236-5741
** ---------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#ifndef SEGYLIBH_H
#define SEGYLIBH_H
#include "segylibh.h"
#endif

FILE *outputFile;

/*
** --------------------------------------------------------------------
** GetBytes: returns translation of a 2 or 4 byte value as a long
**	     integer.
**
** 1) file is opened BINARY.
** 2) User is responsible for positioning the file pointer & retrieveing 4
**    UNSIGNED bytes from the file.
** --------------------------------------------------------------------
*/

long int GetBytes(unsigned char *strBuffer, int byteSize)
{
  int           i;
  unsigned long temp;

  temp=strBuffer[0];
  for(i=1;i<byteSize;i++)
     {
       temp<<=LEFT_SHIFT_BITS;
       temp+=strBuffer[i];
     }
  return(temp);
}

/*
** ---------------------------------------------------------------------
** GetTraceSamples: loads the trace sample array with sample values for
**                  all samples in a given trace.
**
** 1) gets sample values for each sample within a trace
** 2) calculates the average non-zero sample value for a given trace.
** 3) returns FALSE if there is a problem reading a trace sample.
** 4) returns TRUE upon successful completion.
** ---------------------------------------------------------------------
*/

int GetTraceSamples(FILE *pIn, TRCSAMPLES *samples)
{
  int           signBit,
		i,
		sumCount=0,
		exponent;
  double        sumSamples=0.0;
  unsigned char inputBuffer[4];

  samples->avgValue=0.0;
  for(i=0;i<samples->numberOfSamples;i++)
  {
    if((fread((char *)inputBuffer,sizeof(char),L_BUFF_SIZE,pIn))
       <L_BUFF_SIZE)
      return(FALSE);

    samples->values[i]=CalcSampleValue(inputBuffer);

    if(samples->values[i]!=0.0)
      {
	if(samples->values[i]<0.0)
	  sumSamples+=(samples->values[i]*-1.0);
	else
	  sumSamples+=samples->values[i];
	sumCount++;
      }
  }
  if(sumSamples>0.0)
    samples->avgValue=(float)(sumSamples/sumCount);

  return(TRUE);
}

/*
** ---------------------------------------------------------------------
** CalcSampleValue: returns the value calculated for an individual sample
**
** ---------------------------------------------------------------------
*/
float CalcSampleValue(unsigned char buffer[])
{
  int           signBit,
		exponent;
  unsigned int  signMask=0x80,
		exponentMask=0x7f;
  unsigned long temp1=0;
  float         sample=0.0;
  double        mantissa=0.0;

  signBit=(int)(buffer[0]&signMask);

  exponent=(int)(buffer[0]&exponentMask);

  temp1=buffer[1];
  temp1<<=8;
  temp1+=buffer[2];
  temp1<<=8;
  temp1+=buffer[3];

  mantissa=((double)(temp1))/16777216.0; /* 16777216 = 2 raised to the */
					 /* 24th power                 */
  if(signBit==0)
    signBit=1;
  else
    signBit=-1;

  if(mantissa!=0.0)
    sample=(float)(signBit*
		   pow((double)16,(double)(exponent-64))*
		   mantissa);

  return(sample);
}

/*
** -------------------------------------------------------------------
** GoToByte: set file pointer to a specific byte within the SEGY file
**
** 1) user passes: file pointer
**                 byte location in file desired
** 2) returns TRUE if successfull, FALSE if not
** -------------------------------------------------------------------
*/
int GoToByte(FILE *pIn, unsigned long byteNumber)
{
  int i;

  i=fseek(pIn,byteNumber-1,SEEK_SET);
  if(i==0)
    return(TRUE);
  else
    return(FALSE);
}

/*
** ---------------------------------------------------------------------
** GoToTrace: set file pointer to a specific trace within the SEGY file
**
** 1) user passses: file pointer
**                  sequential trace number
**                  bytes per trace
** 2) returns TRUE if successfull, FALSE if not
**
** ---------------------------------------------------------------------
*/
int GoToTrace(FILE *pIn, long traceNumber,int bytesPerTrace)
{
  int i;
  unsigned long offset;

  offset=((traceNumber-1L)*(long)bytesPerTrace+3600L);
  i=fseek(pIn,offset,SEEK_SET);
  if(i==0)
    return(TRUE);
  else
    return(FALSE);
}

/*
** ---------------------------------------------------------------------
** GetCDP: reads 4 byte trace header value & decodes CDP number
**
** 1) user passes: file pointer.
**                 cdp byte location within data file.
** 2) returns TRUE if successful, FALSE if not.
**
** ---------------------------------------------------------------------
*/

int GetCDP(FILE *fp,long byteLoc,long *cdp)
{
  int istat=0;
  char strBuffer[L_BUFF_SIZE];

  istat=GoToByte(fp,byteLoc);
  if(istat==TRUE)
    {
      istat=fread((char *)strBuffer,sizeof(char),L_BUFF_SIZE,fp);
      if(istat==L_BUFF_SIZE)
	{
	  *cdp=GetBytes(strBuffer,L_BUFF_SIZE);
	  istat=TRUE;
	}
      else
	istat=FALSE;
    }

  return(istat);
}

/*
** ---------------------------------------------------------------------
** Get4Bytes: reads 4 byte trace header value & decodes to a long integer
**
** 1) user passes: file pointer.
**                 byte location within data file.
**                 pointer to receive converted value.
** 2) returns TRUE if successful, FALSE if not.
**
** ---------------------------------------------------------------------
*/

int Get4Bytes(FILE *fp,long byteLoc,long *cnvrt)
{
  int istat=0;
  char strBuffer[L_BUFF_SIZE];

  istat=GoToByte(fp,byteLoc);
  if(istat==TRUE)
    {
      istat=fread((char *)strBuffer,sizeof(char),L_BUFF_SIZE,fp);
      if(istat==L_BUFF_SIZE)
	{
	  *cnvrt=GetBytes(strBuffer,L_BUFF_SIZE);
	  istat=TRUE;
	}
      else
	istat=FALSE;
    }

  return(istat);
}

/*
** ---------------------------------------------------------------------
** Get2Bytes: reads 2 byte trace header value & decodes to an integer
**
** 1) user passes: file pointer.
**                 byte location within data file.
**                 pointer to receive converted value.
** 2) returns TRUE if successful, FALSE if not.
**
** ---------------------------------------------------------------------
*/

int Get2Bytes(FILE *fp,long byteLoc,int *cnvrt)
{
  int istat=0;
  char strBuffer[S_BUFF_SIZE];

  istat=GoToByte(fp,byteLoc);
  if(istat==TRUE)
    {
      istat=fread((char *)strBuffer,sizeof(char),S_BUFF_SIZE,fp);
      if(istat==S_BUFF_SIZE)
	{
	  *cnvrt=(int)GetBytes(strBuffer,S_BUFF_SIZE);
	  istat=TRUE;
	}
      else
	istat=FALSE;
    }

  return(istat);
}

/*
** ---------------------------------------------------------------------
** GetSampRate: read 2 byte trace header value, decode sample rate
**              & convert to milliseconds
**
** 1) user passes: file pointer
**                 byte location of sample rate in data file
** 2) returns TRUE if successful, FALSE if not.
**
** ---------------------------------------------------------------------
*/

int GetSampRate(FILE *fp,long byteLoc,float *sampleRate)
{
  int istat=0;
  char strBuffer[S_BUFF_SIZE];

  istat=GoToByte(fp,byteLoc);
  if(istat==TRUE)
    {
      istat=fread((char *)strBuffer,sizeof(char),S_BUFF_SIZE,fp);
      if(istat==S_BUFF_SIZE)
	{
	  *sampleRate=((float)(GetBytes(strBuffer,S_BUFF_SIZE)))/1000.0;
	  istat=TRUE;
	}
      else
	istat=FALSE;
    }
  return(istat);
}

/*
** ---------------------------------------------------------------------
** GetNumberSamples: reads 2 byte trace header value & decodes number of
**                   samples.
**
** 1) user passes: file pointer
**                 byte location of number of samples in data file
** 2) returns TRUE if successful, FALSE if not
**
** ---------------------------------------------------------------------
*/

int GetNumberSamples(FILE *fp,long byteLoc,int *numberSamples)
{
  int istat=0;
  char strBuffer[S_BUFF_SIZE];

  istat=GoToByte(fp,byteLoc);
  if(istat==TRUE)
    {
      istat=fread((char *)strBuffer,sizeof(char),S_BUFF_SIZE,fp);
      if(istat==S_BUFF_SIZE)
	{
	  *numberSamples=(int)GetBytes(strBuffer,S_BUFF_SIZE);
	  istat=TRUE;
	}
      else
	istat=FALSE;
    }
  return(istat);
}

/*
** ------------------------------------------------------------------
** GetEDCDIC: translates 3200 byte EBCDIC header as 80 char ASCII
**            strings
**
** 1) user passes: 3200 byte buffer
** 2) translates buffer bytes into ASCII values
** 3) returns true when done
** ------------------------------------------------------------------
*/

/* EBCDIC TRANSLATION TABLE */
unsigned char EBCDIC_TABLE[256]=
{
	0x00,0x01,0x02,0x03,
	0x00,0x09,0x00,0x7f,
	0x00,0x00,0x00,0x0b,
	0x0c,0x0d,0x0e,0x0f,
	0x10,0x11,0x12,0x00,
	0x00,0x13,0x08,0x00,
	0x18,0x19,0x00,0x00,
	0x1c,0x1d,0x1e,0x1f,
	0x00,0x00,0x1c,0x00,
	0x00,0x0a,0x17,0x1b,
	0x00,0x00,0x00,0x00,
	0x00,0x05,0x06,0x07,
	0x00,0x00,0x16,0x00,
	0x00,0x1e,0x00,0x04,
	0x00,0x00,0x00,0x00,
	0x14,0x15,0x00,0x1a,
	0x20,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x2e,
	0x3c,0x28,0x2b,0x5e,
	0x26,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x21,0x24,
	0x2a,0x29,0x3b,0x5e,
	0x2d,0x2f,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x7c,0x2c,
	0x25,0x5f,0x3e,0x3f,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x60,0x3a,0x23,
	0x40,0x27,0x3d,0x22,
	0x00,0x61,0x62,0x63,
	0x64,0x65,0x66,0x67,
	0x68,0x69,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x6a,0x6b,0x6c,
	0x6d,0x6e,0x6f,0x70,
	0x71,0x72,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x7e,0x73,0x74,
	0x75,0x76,0x77,0x78,
	0x79,0x7a,0x00,0x00,
	0x00,0x5b,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x00,0x5d,0x00,0x00,
	0x7b,0x41,0x42,0x43,
	0x44,0x45,0x46,0x47,
	0x48,0x49,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x7d,0x4a,0x4b,0x4c,
	0x4d,0x4e,0x4f,0x50,
	0x54,0x52,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x5c,0x00,0x53,0x54,
	0x55,0x56,0x57,0x58,
	0x59,0x5a,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x30,0x31,0x32,0x33,
	0x34,0x35,0x36,0x37,
	0x38,0x39,0x00,0x00,
	0x00,0x00,0x00,0x00
};

int GetEBCDICHeader(unsigned char headerBuffer[],EBCDIC *hdr)
{
  int i,j=0,k=0;

  for(i=0;i<3200;i++)
    {
      hdr->table[j][k++]=EBCDIC_TABLE[headerBuffer[i]];
      if(k==80)
	{
	  hdr->table[j][k]='\0';
	  j++;
	  k=0;
	}
    }

  return(TRUE);
}

/*
** ------------------------------------------------------------------
** GetBinaryHeader: translate the 1st 60 bytes of 400 byte binary header
**
** 1) user passes: input buffer
** 2) returns TRUE when done
** ------------------------------------------------------------------
*/

int GetBinaryHeader(unsigned char binaryBuffer[], BINHEADER *hdr)
{
  unsigned int temp;

  hdr->jobID=binaryBuffer[0];
  hdr->jobID<<=8;
  hdr->jobID+=binaryBuffer[1];
  hdr->jobID<<=8;
  hdr->jobID+=binaryBuffer[2];
  hdr->jobID<<=8;
  hdr->jobID+=binaryBuffer[3];

  hdr->lineNumber=binaryBuffer[4];
  hdr->lineNumber<<=8;
  hdr->lineNumber+=binaryBuffer[5];
  hdr->lineNumber<<=8;
  hdr->lineNumber+=binaryBuffer[6];
  hdr->lineNumber<<=8;
  hdr->lineNumber+=binaryBuffer[7];

  hdr->reelNumber=binaryBuffer[8];
  hdr->reelNumber<<=8;
  hdr->reelNumber+=binaryBuffer[9];
  hdr->reelNumber<<=8;
  hdr->reelNumber+=binaryBuffer[10];
  hdr->reelNumber<<=8;
  hdr->reelNumber+=binaryBuffer[11];

  hdr->tracesPerRecord=binaryBuffer[12];
  hdr->tracesPerRecord<<=8;
  hdr->tracesPerRecord+=binaryBuffer[13];

  hdr->auxTracesPerRecord=binaryBuffer[14];
  hdr->auxTracesPerRecord<<=8;
  hdr->auxTracesPerRecord+=binaryBuffer[15];


  hdr->sampleRateReel=binaryBuffer[16];
  hdr->sampleRateReel<<=8;
  hdr->sampleRateReel+=binaryBuffer[17];

  hdr->sampleRateOrig=binaryBuffer[18];
  hdr->sampleRateOrig<<=8;
  hdr->sampleRateOrig+=binaryBuffer[19];

  hdr->samplesPerTraceReel=binaryBuffer[20];
  hdr->samplesPerTraceReel<<=8;
  hdr->samplesPerTraceReel+=binaryBuffer[21];

  hdr->samplesPerTraceOrig=binaryBuffer[22];
  hdr->samplesPerTraceOrig<<=8;
  hdr->samplesPerTraceOrig+=binaryBuffer[23];

  hdr->dataFormatCode=binaryBuffer[24];
  hdr->dataFormatCode<<=8;
  hdr->dataFormatCode+=binaryBuffer[25];

  hdr->CDPfold=binaryBuffer[26];
  hdr->CDPfold<<=8;
  hdr->CDPfold+=binaryBuffer[27];

  hdr->traceSortCode=binaryBuffer[28];
  hdr->traceSortCode<<=8;
  hdr->traceSortCode+=binaryBuffer[29];

  hdr->verticalSumCode=binaryBuffer[30];
  hdr->verticalSumCode<<=8;
  hdr->verticalSumCode+=binaryBuffer[31];

  hdr->sweepFreqStart=binaryBuffer[32];
  hdr->sweepFreqStart<<=8;
  hdr->sweepFreqStart+=binaryBuffer[33];

  hdr->sweepFreqEnd=binaryBuffer[34];
  hdr->sweepFreqEnd<<=8;
  hdr->sweepFreqEnd+=binaryBuffer[35];

  hdr->sweepLength=binaryBuffer[36];
  hdr->sweepLength<<=8;
  hdr->sweepLength+=binaryBuffer[37];

  hdr->sweepTypeCode=binaryBuffer[38];
  hdr->sweepTypeCode<<=8;
  hdr->sweepTypeCode+=binaryBuffer[39];

  hdr->sweepChannelTraceNumber=binaryBuffer[40];
  hdr->sweepChannelTraceNumber<<=8;
  hdr->sweepChannelTraceNumber+=binaryBuffer[41];

  hdr->sweepTraceTaperLengthStart=binaryBuffer[42];
  hdr->sweepTraceTaperLengthStart<<=8;
  hdr->sweepTraceTaperLengthStart+=binaryBuffer[43];

  hdr->sweepTraceTaperLengthEnd=binaryBuffer[44];
  hdr->sweepTraceTaperLengthEnd<<=8;
  hdr->sweepTraceTaperLengthEnd+=binaryBuffer[45];

  hdr->taperType=binaryBuffer[46];
  hdr->taperType<<=8;
  hdr->taperType+=binaryBuffer[47];

  hdr->correlatedData=binaryBuffer[48];
  hdr->correlatedData<<=8;
  hdr->correlatedData+=binaryBuffer[49];

  hdr->binaryGainRecovered=binaryBuffer[50];
  hdr->binaryGainRecovered<<=8;
  hdr->binaryGainRecovered+=binaryBuffer[51];

  hdr->amplitudeRecovery=binaryBuffer[52];
  hdr->amplitudeRecovery<<=8;
  hdr->amplitudeRecovery+=binaryBuffer[53];

  hdr->measurementSystem=binaryBuffer[54];
  hdr->measurementSystem<<=8;
  hdr->measurementSystem+=binaryBuffer[55];

  hdr->impulseSignalPolarity=binaryBuffer[56];
  hdr->impulseSignalPolarity<<=8;
  hdr->impulseSignalPolarity+=binaryBuffer[57];

  hdr->vibratoryPolarityCode=binaryBuffer[58];
  hdr->vibratoryPolarityCode<<=8;
  hdr->vibratoryPolarityCode+=binaryBuffer[59];

  return(TRUE);
}

/*
** ------------------------------------------------------------------
** GetTraceHeader: decodes trace header values from 240 byte unsigned
**                 char trcHeaderBuffer
**
** 1) user passes: trace header buffer array
** 2) returns TRUE when done
** ------------------------------------------------------------------
*/
int GetTraceHeader(unsigned char trcHeaderBuffer[], TRCHEADER *hdr)
{
  unsigned int temp;

  hdr->traceSequenceNumberLine=trcHeaderBuffer[0];
  hdr->traceSequenceNumberLine<<=8;
  hdr->traceSequenceNumberLine+=trcHeaderBuffer[1];
  hdr->traceSequenceNumberLine<<=8;
  hdr->traceSequenceNumberLine+=trcHeaderBuffer[2];
  hdr->traceSequenceNumberLine<<=8;
  hdr->traceSequenceNumberLine+=trcHeaderBuffer[3];

  hdr->traceSequenceNumberReel=trcHeaderBuffer[4];
  hdr->traceSequenceNumberReel<<=8;
  hdr->traceSequenceNumberReel+=trcHeaderBuffer[5];
  hdr->traceSequenceNumberReel<<=8;
  hdr->traceSequenceNumberReel+=trcHeaderBuffer[6];
  hdr->traceSequenceNumberReel<<=8;
  hdr->traceSequenceNumberReel+=trcHeaderBuffer[7];

  hdr->origFieldRecordNumber=trcHeaderBuffer[8];
  hdr->origFieldRecordNumber<<=8;
  hdr->origFieldRecordNumber+=trcHeaderBuffer[9];
  hdr->origFieldRecordNumber<<=8;
  hdr->origFieldRecordNumber+=trcHeaderBuffer[10];
  hdr->origFieldRecordNumber<<=8;
  hdr->origFieldRecordNumber+=trcHeaderBuffer[11];

  hdr->fieldRecordTraceNumber=trcHeaderBuffer[12];
  hdr->fieldRecordTraceNumber<<=8;
  hdr->fieldRecordTraceNumber+=trcHeaderBuffer[13];
  hdr->fieldRecordTraceNumber<<=8;
  hdr->fieldRecordTraceNumber+=trcHeaderBuffer[14];
  hdr->fieldRecordTraceNumber<<=8;
  hdr->fieldRecordTraceNumber+=trcHeaderBuffer[15];

  hdr->energySourcePt=trcHeaderBuffer[16];
  hdr->energySourcePt<<=8;
  hdr->energySourcePt+=trcHeaderBuffer[17];
  hdr->energySourcePt<<=8;
  hdr->energySourcePt+=trcHeaderBuffer[18];
  hdr->energySourcePt<<=8;
  hdr->energySourcePt+=trcHeaderBuffer[19];

  hdr->cdpNumber=trcHeaderBuffer[20];
  hdr->cdpNumber<<=8;
  hdr->cdpNumber+=trcHeaderBuffer[21];
  hdr->cdpNumber<<=8;
  hdr->cdpNumber+=trcHeaderBuffer[22];
  hdr->cdpNumber<<=8;
  hdr->cdpNumber+=trcHeaderBuffer[23];

  hdr->traceCdpNumber=trcHeaderBuffer[24];
  hdr->traceCdpNumber<<=8;
  hdr->traceCdpNumber+=trcHeaderBuffer[25];
  hdr->traceCdpNumber<<=8;
  hdr->traceCdpNumber+=trcHeaderBuffer[26];
  hdr->traceCdpNumber<<=8;
  hdr->traceCdpNumber+=trcHeaderBuffer[27];

  hdr->traceIdNumber=trcHeaderBuffer[28];
  hdr->traceIdNumber<<=8;
  hdr->traceIdNumber+=trcHeaderBuffer[29];

  hdr->verticallySummedTraces=trcHeaderBuffer[30];
  hdr->verticallySummedTraces<<=8;
  hdr->verticallySummedTraces+=trcHeaderBuffer[31];

  hdr->horizontallyStackedTraces=trcHeaderBuffer[32];
  hdr->horizontallyStackedTraces<<=8;
  hdr->horizontallyStackedTraces+=trcHeaderBuffer[33];

  hdr->dataUse=trcHeaderBuffer[34];
  hdr->dataUse<<=8;
  hdr->dataUse+=trcHeaderBuffer[35];

  hdr->sourceToReceiver=trcHeaderBuffer[36];
  hdr->sourceToReceiver<<=8;
  hdr->sourceToReceiver+=trcHeaderBuffer[37];
  hdr->sourceToReceiver<<=8;
  hdr->sourceToReceiver+=trcHeaderBuffer[38];
  hdr->sourceToReceiver<<=8;
  hdr->sourceToReceiver+=trcHeaderBuffer[39];

  hdr->receiverElevation=trcHeaderBuffer[40];
  hdr->receiverElevation<<=8;
  hdr->receiverElevation+=trcHeaderBuffer[41];
  hdr->receiverElevation<<=8;
  hdr->receiverElevation+=trcHeaderBuffer[42];
  hdr->receiverElevation<<=8;
  hdr->receiverElevation+=trcHeaderBuffer[43];

  hdr->sourceSurfaceElevation=trcHeaderBuffer[44];
  hdr->sourceSurfaceElevation<<=8;
  hdr->sourceSurfaceElevation+=trcHeaderBuffer[45];
  hdr->sourceSurfaceElevation<<=8;
  hdr->sourceSurfaceElevation+=trcHeaderBuffer[46];
  hdr->sourceSurfaceElevation<<=8;
  hdr->sourceSurfaceElevation+=trcHeaderBuffer[47];

  hdr->sourceDepth=trcHeaderBuffer[48];
  hdr->sourceDepth<<=8;
  hdr->sourceDepth+=trcHeaderBuffer[49];
  hdr->sourceDepth<<=8;
  hdr->sourceDepth+=trcHeaderBuffer[50];
  hdr->sourceDepth<<=8;
  hdr->sourceDepth+=trcHeaderBuffer[51];

  hdr->receiverDatumElevation=trcHeaderBuffer[52];
  hdr->receiverDatumElevation<<=8;
  hdr->receiverDatumElevation+=trcHeaderBuffer[53];
  hdr->receiverDatumElevation<<=8;
  hdr->receiverDatumElevation+=trcHeaderBuffer[54];
  hdr->receiverDatumElevation<<=8;
  hdr->receiverDatumElevation+=trcHeaderBuffer[55];

  hdr->sourceDatumElevation=trcHeaderBuffer[56];
  hdr->sourceDatumElevation<<=8;
  hdr->sourceDatumElevation+=trcHeaderBuffer[57];
  hdr->sourceDatumElevation<<=8;
  hdr->sourceDatumElevation+=trcHeaderBuffer[58];
  hdr->sourceDatumElevation<<=8;
  hdr->sourceDatumElevation+=trcHeaderBuffer[59];

  hdr->sourceWaterDepth=trcHeaderBuffer[60];
  hdr->sourceWaterDepth<<=8;
  hdr->sourceWaterDepth+=trcHeaderBuffer[61];
  hdr->sourceWaterDepth<<=8;
  hdr->sourceWaterDepth+=trcHeaderBuffer[62];
  hdr->sourceWaterDepth<<=8;
  hdr->sourceWaterDepth+=trcHeaderBuffer[63];

  hdr->groupWaterDepth=trcHeaderBuffer[64];
  hdr->groupWaterDepth<<=8;
  hdr->groupWaterDepth+=trcHeaderBuffer[65];
  hdr->groupWaterDepth<<=8;
  hdr->groupWaterDepth+=trcHeaderBuffer[66];
  hdr->groupWaterDepth<<=8;
  hdr->groupWaterDepth+=trcHeaderBuffer[67];

  hdr->elevationScalar=trcHeaderBuffer[68];
  hdr->elevationScalar<<=8;
  hdr->elevationScalar+=trcHeaderBuffer[69];

  hdr->coordinateScalar=trcHeaderBuffer[70];
  hdr->coordinateScalar<<=8;
  hdr->coordinateScalar+=trcHeaderBuffer[71];

  hdr->sourceXCoordinate=trcHeaderBuffer[72];
  hdr->sourceXCoordinate<<=8;
  hdr->sourceXCoordinate+=trcHeaderBuffer[73];
  hdr->sourceXCoordinate<<=8;
  hdr->sourceXCoordinate+=trcHeaderBuffer[74];
  hdr->sourceXCoordinate<<=8;
  hdr->sourceXCoordinate+=trcHeaderBuffer[75];

  hdr->sourceYCoordinate=trcHeaderBuffer[76];
  hdr->sourceYCoordinate<<=8;
  hdr->sourceYCoordinate+=trcHeaderBuffer[77];
  hdr->sourceYCoordinate<<=8;
  hdr->sourceYCoordinate+=trcHeaderBuffer[78];
  hdr->sourceYCoordinate<<=8;
  hdr->sourceYCoordinate+=trcHeaderBuffer[79];

  hdr->groupXCoordinate=trcHeaderBuffer[80];
  hdr->groupXCoordinate<<=8;
  hdr->groupXCoordinate+=trcHeaderBuffer[81];
  hdr->groupXCoordinate<<=8;
  hdr->groupXCoordinate+=trcHeaderBuffer[82];
  hdr->groupXCoordinate<<=8;
  hdr->groupXCoordinate+=trcHeaderBuffer[83];

  hdr->groupYCoordinate=trcHeaderBuffer[84];
  hdr->groupYCoordinate<<=8;
  hdr->groupYCoordinate+=trcHeaderBuffer[85];
  hdr->groupYCoordinate<<=8;
  hdr->groupYCoordinate+=trcHeaderBuffer[86];
  hdr->groupYCoordinate<<=8;
  hdr->groupYCoordinate+=trcHeaderBuffer[87];

  hdr->coordinateUnits=trcHeaderBuffer[88];
  hdr->coordinateUnits<<=8;
  hdr->coordinateUnits+=trcHeaderBuffer[89];

  hdr->weatheringVelocity=trcHeaderBuffer[90];
  hdr->weatheringVelocity<<=8;
  hdr->weatheringVelocity+=trcHeaderBuffer[91];

  hdr->subWeatheringVelocity=trcHeaderBuffer[92];
  hdr->subWeatheringVelocity<<=8;
  hdr->subWeatheringVelocity+=trcHeaderBuffer[93];

  hdr->sourceUpholeTime=trcHeaderBuffer[94];
  hdr->sourceUpholeTime<<=8;
  hdr->sourceUpholeTime+=trcHeaderBuffer[95];

  hdr->groupUpholeTime=trcHeaderBuffer[96];
  hdr->groupUpholeTime<<=8;
  hdr->groupUpholeTime+=trcHeaderBuffer[97];

  hdr->sourceStaticCorrection=trcHeaderBuffer[98];
  hdr->sourceStaticCorrection<<=8;
  hdr->sourceStaticCorrection+=trcHeaderBuffer[99];

  hdr->groupStaticCorrection=trcHeaderBuffer[100];
  hdr->groupStaticCorrection<<=8;
  hdr->groupStaticCorrection+=trcHeaderBuffer[101];

  hdr->totalStaticApplied=trcHeaderBuffer[102];
  hdr->totalStaticApplied<<=8;
  hdr->totalStaticApplied+=trcHeaderBuffer[103];

  hdr->lagTimeA=trcHeaderBuffer[104];
  hdr->lagTimeA<<=8;
  hdr->lagTimeA+=trcHeaderBuffer[105];

  hdr->lagTimeB=trcHeaderBuffer[106];
  hdr->lagTimeB<<=8;
  hdr->lagTimeB+=trcHeaderBuffer[107];

  hdr->delayRecordingTime=trcHeaderBuffer[108];
  hdr->delayRecordingTime<<=8;
  hdr->delayRecordingTime+=trcHeaderBuffer[109];

  hdr->muteTimeStart=trcHeaderBuffer[110];
  hdr->muteTimeStart<<=8;
  hdr->muteTimeStart+=trcHeaderBuffer[111];

  hdr->muteTimeEnd=trcHeaderBuffer[112];
  hdr->muteTimeEnd<<=8;
  hdr->muteTimeEnd+=trcHeaderBuffer[113];

  hdr->numberOfSamples=trcHeaderBuffer[114];
  hdr->numberOfSamples<<=8;
  hdr->numberOfSamples+=trcHeaderBuffer[115];

  hdr->sampleInterval=trcHeaderBuffer[116];
  hdr->sampleInterval<<=8;
  hdr->sampleInterval+=trcHeaderBuffer[117];

  hdr->gainType=trcHeaderBuffer[118];
  hdr->gainType<<=8;
  hdr->gainType+=trcHeaderBuffer[119];

  hdr->gainConstant=trcHeaderBuffer[120];
  hdr->gainConstant<<=8;
  hdr->gainConstant+=trcHeaderBuffer[121];

  hdr->initialGain=trcHeaderBuffer[122];
  hdr->initialGain<<=8;
  hdr->initialGain+=trcHeaderBuffer[123];

  hdr->correlated=trcHeaderBuffer[124];
  hdr->correlated<<=8;
  hdr->correlated+=trcHeaderBuffer[125];

  hdr->sweepFrequencyStart=trcHeaderBuffer[126];
  hdr->sweepFrequencyStart<<=8;
  hdr->sweepFrequencyStart+=trcHeaderBuffer[127];

  hdr->sweepFrequencyEnd=trcHeaderBuffer[128];
  hdr->sweepFrequencyEnd<<=8;
  hdr->sweepFrequencyEnd+=trcHeaderBuffer[129];

  hdr->sweepLength=trcHeaderBuffer[130];
  hdr->sweepLength<<=8;
  hdr->sweepLength+=trcHeaderBuffer[131];

  hdr->sweepType=trcHeaderBuffer[132];
  hdr->sweepType<<=8;
  hdr->sweepType+=trcHeaderBuffer[133];

  hdr->sweepTraceTaperStart=trcHeaderBuffer[134];
  hdr->sweepTraceTaperStart<<=8;
  hdr->sweepTraceTaperStart+=trcHeaderBuffer[135];

  hdr->sweepTraceTaperEnd=trcHeaderBuffer[136];
  hdr->sweepTraceTaperEnd<<=8;
  hdr->sweepTraceTaperEnd+=trcHeaderBuffer[137];

  hdr->taperType=trcHeaderBuffer[138];
  hdr->taperType<<=8;
  hdr->taperType+=trcHeaderBuffer[139];

  hdr->aliasFilterFrequency=trcHeaderBuffer[140];
  hdr->aliasFilterFrequency<<=8;
  hdr->aliasFilterFrequency+=trcHeaderBuffer[141];

  hdr->aliasFilterSlope=trcHeaderBuffer[142];
  hdr->aliasFilterSlope<<=8;
  hdr->aliasFilterSlope+=trcHeaderBuffer[143];

  hdr->notchFilterFrequency=trcHeaderBuffer[144];
  hdr->notchFilterFrequency<<=8;
  hdr->notchFilterFrequency+=trcHeaderBuffer[145];

  hdr->notchFilterSlope=trcHeaderBuffer[146];
  hdr->notchFilterSlope<<=8;
  hdr->notchFilterSlope+=trcHeaderBuffer[147];

  hdr->lowCutFrequency=trcHeaderBuffer[148];
  hdr->lowCutFrequency<<=8;
  hdr->lowCutFrequency+=trcHeaderBuffer[149];

  hdr->highCutFrequency=trcHeaderBuffer[150];
  hdr->highCutFrequency<<=8;
  hdr->highCutFrequency+=trcHeaderBuffer[151];

  hdr->lowCutSlope=trcHeaderBuffer[152];
  hdr->lowCutSlope<<=8;
  hdr->lowCutSlope+=trcHeaderBuffer[153];

  hdr->highCutSlope=trcHeaderBuffer[154];
  hdr->highCutSlope<<=8;
  hdr->highCutSlope+=trcHeaderBuffer[155];

  hdr->yearRecorded=trcHeaderBuffer[156];
  hdr->yearRecorded<<=8;
  hdr->yearRecorded+=trcHeaderBuffer[157];

  hdr->dayOfYear=trcHeaderBuffer[158];
  hdr->dayOfYear<<=8;
  hdr->dayOfYear+=trcHeaderBuffer[159];

  hdr->hourOfDay=trcHeaderBuffer[160];
  hdr->hourOfDay<<=8;
  hdr->hourOfDay+=trcHeaderBuffer[161];

  hdr->minuteOfHour=trcHeaderBuffer[162];
  hdr->minuteOfHour<<=8;
  hdr->minuteOfHour+=trcHeaderBuffer[163];

  hdr->secondOfMinute=trcHeaderBuffer[164];
  hdr->secondOfMinute<<=8;
  hdr->secondOfMinute+=trcHeaderBuffer[165];

  hdr->timeBasisCode=trcHeaderBuffer[166];
  hdr->timeBasisCode<<=8;
  hdr->timeBasisCode+=trcHeaderBuffer[167];

  hdr->traceWeightingFactor=trcHeaderBuffer[168];
  hdr->traceWeightingFactor<<=8;
  hdr->traceWeightingFactor+=trcHeaderBuffer[169];

  hdr->geophoneNumberRollSwitch1=trcHeaderBuffer[58];
  hdr->geophoneNumberRollSwitch1<<=8;
  hdr->geophoneNumberRollSwitch1+=trcHeaderBuffer[59];

  hdr->geophoneNumberTrace1=trcHeaderBuffer[58];
  hdr->geophoneNumberTrace1<<=8;
  hdr->geophoneNumberTrace1+=trcHeaderBuffer[59];

  hdr->geophoneNumberLastTrace=trcHeaderBuffer[58];
  hdr->geophoneNumberLastTrace<<=8;
  hdr->geophoneNumberLastTrace+=trcHeaderBuffer[59];

  hdr->gapSize=trcHeaderBuffer[58];
  hdr->gapSize<<=8;
  hdr->gapSize+=trcHeaderBuffer[59];

  hdr->overTravel=trcHeaderBuffer[58];
  hdr->overTravel<<=8;
  hdr->overTravel+=trcHeaderBuffer[59];

  return(TRUE);
}